package com.jiuqi.grid;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.jiuqi.grid.CellDataProperty;
import com.jiuqi.grid.CellDataPropertyIntf;
import com.jiuqi.grid.CellField;
import com.jiuqi.grid.CurrencyCellProperty;
import com.jiuqi.grid.CurrencyCellPropertyIntf;
import com.jiuqi.grid.GridData;
import com.jiuqi.grid.NumberCellProperty;
import com.jiuqi.grid.NumberCellPropertyIntf;
import com.jiuqi.grid.TextCellProperty;
import com.jiuqi.grid.TextCellPropertyIntf;
import com.jiuqi.util.Base64;
import com.jiuqi.util.DateUtil;
import com.jiuqi.util.JqLib;
import com.jiuqi.util.MemStream;
import com.jiuqi.util.Stream;
import com.jiuqi.util.StreamException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.Date;

public class GridCell implements Serializable {
  private int col;
  
  private int row;
  
  private byte[] data = new byte[20];
  
  protected GridData grid;
  
  protected void dataChanged() {}
  
  public static int BGR2RGB(int color) {
    if (color < 0)
      return color; 
    int red = (color & 0xFF) << 16;
    int blue = (color & 0xFF0000) >> 16;
    int green = color & 0xFF00;
    return red | blue | green;
  }
  
  public static int RGB2BGR(int color) {
    if (color < 0)
      return color; 
    int red = (color & 0xFF0000) >> 16;
    int blue = (color & 0xFF) << 16;
    int green = color & 0xFF00;
    return red | blue | green;
  }
  
  private boolean getBit(int bytePos, int bitPos) {
    return ((this.data[bytePos] >> bitPos & 0x1) > 0);
  }
  
  private void setBit(int bytePos, int bitPos, boolean value) {
    byte flag = (byte)(1 << bitPos);
    if (value) {
      this.data[bytePos] = (byte)(this.data[bytePos] | flag);
    } else {
      this.data[bytePos] = (byte)(this.data[bytePos] & (flag ^ 0xFFFFFFFF));
    } 
  }
  
  private static int byteToInt(byte b) {
    return (b < 0) ? (127 + Math.abs(b)) : b;
  }
  
  public final void init(GridData g, int aCol, int aRow) {
    init(g, aCol, aRow, g.getCellPropData(aCol, aRow));
  }
  
  public final void init(GridData g, int aCol, int aRow, byte[] celldata) {
    System.arraycopy(celldata, 0, this.data, 0, celldata.length);
    this.grid = g;
    this.col = aCol;
    this.row = aRow;
  }
  
  public final byte[] getPropData() {
    return this.data;
  }
  
  public final int getBackColor() {
    return BGR2RGB(this.grid.cellColors().get(byteToInt(this.data[0])));
  }
  
  public final boolean getBackLines(int cellBackStyle) {
    if (cellBackStyle >= 0 && cellBackStyle <= 3)
      return getBit(1, cellBackStyle + 4); 
    return false;
  }
  
  public final int getBackStyle() {
    return this.data[2] >> 0 & 0x3F;
  }
  
  public final int getBEdgeColor() {
    return BGR2RGB(this.grid.edgeColors().get(this.data[10] >> 4 & 0xF));
  }
  
  public final int getBEdgeStyle() {
    return this.data[10] >> 0 & 0xF;
  }
  
  public final boolean getCanInput() {
    return getBit(2, 7);
  }
  
  public final boolean getCanModify() {
    return getBit(2, 6);
  }
  
  public final boolean getCanRead() {
    return getBit(4, 7);
  }
  
  public final boolean getCanSelect() {
    return getBit(4, 6);
  }
  
  public final boolean getCanWrite() {
    return getBit(5, 7);
  }
  
  public final int getDataType() {
    return this.data[3] >> 4 & 0xF;
  }
  
  public final boolean getEditJumpNext() {
    return getBit(5, 6);
  }
  
  public final String getEditMask() {
    return null;
  }
  
  public final int getEditMode() {
    return this.data[3] >> 0 & 0xF;
  }
  
  public final boolean getEmptyCell() {
    return getBit(6, 7);
  }
  
  public final boolean getFitFontSize() {
    return getBit(6, 6);
  }
  
  public final boolean getFontBold() {
    return getBit(7, 7);
  }
  
  public final int getFontColor() {
    int index = this.data[6] >> 0 & 0x3F;
    return BGR2RGB(this.grid.fontColors().get(index));
  }
  
  public final int getFontHeight() {
    return getFontSize() * 96 / 72;
  }
  
  public final boolean getFontItalic() {
    return getBit(7, 6);
  }
  
  public final String getFontName() {
    int index = this.data[4] & 0x3F;
    return this.grid.fontNames().getFont(index);
  }
  
  public final int getFontSize() {
    int index = this.data[5] & 0x3F;
    int h = this.grid.fontSizes().get(index);
    return h;
  }
  
  public final boolean getFontStrikeOut() {
    return getBit(7, 5);
  }
  
  public final boolean getFontStyle(int fontStyle) {
    if (fontStyle >= 0 && fontStyle < 4)
      return getBit(1, fontStyle); 
    return false;
  }
  
  public final boolean getFontUnderLine() {
    return getBit(7, 4);
  }
  
  public final int getHorzAlign() {
    return this.data[7] >> 0 & 0x7;
  }
  
  public final int getImeMode() {
    return this.data[11] >> 0 & 0x7;
  }
  
  public final int getIndent() {
    return this.data[1] >> 0 & 0xF;
  }
  
  public final int getLEdgeColor() {
    return (this.col == 0) ? 0 : this.grid.getCellEx(this.col - 1, this.row).getREdgeColor();
  }
  
  public final int getLEdgeStyle() {
    return (this.col == 0) ? 0 : this.grid.getCellEx(this.col - 1, this.row).getREdgeStyle();
  }
  
  public final boolean getMultiLine() {
    return getBit(8, 7);
  }
  
  public final boolean getReadOnly() {
    return !(getCanModify() && getCanWrite());
  }
  
  public final int getREdgeColor() {
    int index = this.data[9] >> 4 & 0xF;
    return BGR2RGB(this.grid.edgeColors().get(index));
  }
  
  public final int getREdgeStyle() {
    return this.data[9] >> 0 & 0xF;
  }
  
  public final String getShowFormat() {
    return null;
  }
  
  public final boolean getSilverHead() {
    return getBit(8, 6);
  }
  
  public final int getTEdgeColor() {
    return (this.row == 0) ? 0 : this.grid.getCellEx(this.col, this.row - 1).getBEdgeColor();
  }
  
  public final int getTEdgeStyle() {
    return (this.row == 0) ? 0 : this.grid.getCellEx(this.col, this.row - 1).getBEdgeStyle();
  }
  
  public final int getVertAlign() {
    return this.data[8] >> 0 & 0x7;
  }
  
  public final boolean getVertText() {
    return getBit(8, 5);
  }
  
  public final boolean getWrapLine() {
    return getBit(8, 4);
  }
  
  public final boolean getBoolean() {
    return JqLib.isBoolTrue(this.grid.getCellData(this.col, this.row));
  }
  
  public final Date getDate() {
    return DateUtil.tryParserDateString(this.grid.getCellData(this.col, this.row));
  }
  
  public final Date getDateTime() {
    return getDate();
  }
  
  public final double getFloat() {
    String strData = this.grid.getCellData(this.col, this.row);
    if (strData == null)
      return 0.0D; 
    StringBuffer buffer = new StringBuffer();
    for (int i = 0; i < strData.length(); i++) {
      char ch = strData.charAt(i);
      if (Character.isDigit(ch) || ch == '.' || ch == '-' || ch == 'E' || ch == 'e')
        buffer.append(ch); 
    } 
    if (buffer.length() == 0)
      return 0.0D; 
    return Double.parseDouble(buffer.toString());
  }
  
  public final int getInteger() {
    String strData = this.grid.getCellData(this.col, this.row);
    if (strData == null)
      return 0; 
    StringBuffer buffer = new StringBuffer();
    for (int i = 0; i < strData.length(); i++) {
      char ch = strData.charAt(i);
      if (Character.isDigit(ch) || ch == '-') {
        buffer.append(ch);
      } else if (ch == '.') {
        break;
      } 
    } 
    if (buffer.length() == 0)
      return 0; 
    return Integer.parseInt(buffer.toString());
  }
  
  public final String getString() {
    return this.grid.getCellData(this.col, this.row);
  }
  
  public final double getTime() {
    return getFloat();
  }
  
  public final int getCellBottom() {
    CellField cf = this.grid.expandCell(this.col, this.row);
    return cf.bottom;
  }
  
  public final String getCellData() {
    return this.grid.getCellData(this.col, this.row);
  }
  
  public final CellField getCellField() {
    return this.grid.expandCell(this.col, this.row);
  }
  
  public final int getCellHeight() {
    CellField cf = this.grid.expandCell(this.col, this.row);
    int rt = 0;
    for (int i = cf.top; i <= cf.bottom; i++)
      rt += this.grid.getRowHeights(i); 
    return rt;
  }
  
  public final int getCellLeft() {
    return (this.grid.expandCell(this.col, this.row)).left;
  }
  
  public final int getCellRight() {
    return (this.grid.expandCell(this.col, this.row)).right;
  }
  
  public final int getCellTop() {
    return (this.grid.expandCell(this.col, this.row)).top;
  }
  
  public final int getCellWidth() {
    CellField cf = this.grid.expandCell(this.col, this.row);
    int rt = 0;
    for (int i = cf.left; i <= cf.right; i++)
      rt += this.grid.getColWidths(i); 
    return rt;
  }
  
  public final int getColNum() {
    return this.col;
  }
  
  public final int getColSpan() {
    CellField cf = this.grid.expandCell(this.col, this.row);
    return (cf.left == this.col && cf.top == this.row) ? (cf.right - cf.left + 1) : 0;
  }
  
  public final int getColWidth() {
    return this.grid.getColWidths(this.col);
  }
  
  public final boolean getHasConjection() {
    CellField cf = this.grid.expandCell(this.col, this.row);
    return !(cf.left == cf.right && cf.top == cf.bottom);
  }
  
  public final boolean getIsNull() {
    return (this.grid.getCellData(this.col, this.row) == null);
  }
  
  public final boolean getIsZero() {
    return getIsNull() | ((getInteger() == 0));
  }
  
  public final int getRowHeight() {
    return this.grid.getRowHeights(this.row);
  }
  
  public final int getRowNum() {
    return this.row;
  }
  
  public final int getRowSpan() {
    CellField cf = this.grid.expandCell(this.col, this.row);
    return (cf.left == this.col && cf.top == this.row) ? (cf.bottom - cf.top + 1) : 0;
  }
  
  public final boolean getVisible() {
    if (!this.grid.getColVisible(this.col) || !this.grid.getRowVisible(this.row))
      return false; 
    CellField cf = this.grid.expandCell(this.col, this.row);
    return (cf.left == this.col && cf.top == this.row);
  }
  
  public final int getOutputKind() {
    return this.data[11] >> 4 & 0x3;
  }
  
  public final void setBackColor(int value) {
    int color = RGB2BGR(value);
    int index = this.grid.cellColors().indexOf(color);
    if (index < 0)
      if (this.grid.cellColors().count() < 127) {
        index = this.grid.cellColors().add(color);
      } else {
        index = 0;
      }  
    this.data[0] = (byte)index;
    dataChanged();
  }
  
  public final void setBackLines(int cellBackStyle, boolean value) {
    if (cellBackStyle >= 0 && cellBackStyle <= 3)
      setBit(1, cellBackStyle + 4, value); 
    dataChanged();
  }
  
  public final void setBackStyle(int value) {
    this.data[2] = (byte)(this.data[2] & 0xC0 | (byte)value & 0x3F);
    if (value < 28)
      setBit(8, 6, false); 
    dataChanged();
  }
  
  public final void setBEdgeColor(int value) {
    int color = RGB2BGR(value);
    int index = this.grid.edgeColors().indexOf(color);
    if (index < 0)
      if (this.grid.edgeColors().count() < 15) {
        index = this.grid.edgeColors().add(color);
      } else {
        index = 0;
      }  
    this.data[10] = (byte)(this.data[10] & 0xF | index << 4 & 0xF0);
    dataChanged();
  }
  
  public final void setBEdgeStyle(int value) {
    this.data[10] = (byte)(this.data[10] & 0xF0 | value & 0xF);
    dataChanged();
  }
  
  public final void setCanInput(boolean value) {
    setBit(2, 7, value);
    dataChanged();
  }
  
  public final void setCanModify(boolean value) {
    setBit(2, 6, value);
    dataChanged();
  }
  
  public final void setCanRead(boolean value) {
    setBit(4, 7, value);
    dataChanged();
  }
  
  public final void setCanSelect(boolean value) {
    setBit(4, 6, value);
    dataChanged();
  }
  
  public final void setCanWrite(boolean value) {
    setBit(5, 7, value);
    dataChanged();
  }
  
  public final void setDataType(int value) {
    this.data[3] = (byte)(this.data[3] & 0xF | value << 4 & 0xF0);
    dataChanged();
  }
  
  public final void setEditJumpNext(boolean value) {
    setBit(5, 6, value);
    dataChanged();
  }
  
  public final void setEditMask(String value) {}
  
  public final void setEditMode(int value) {
    this.data[3] = (byte)(this.data[3] & 0xF0 | value & 0xF);
    dataChanged();
  }
  
  public final void setEmptyCell(boolean value) {
    setBit(6, 7, value);
    dataChanged();
  }
  
  public final void setFitFontSize(boolean value) {
    setBit(6, 6, value);
    dataChanged();
  }
  
  public final void setFontBold(boolean value) {
    setBit(7, 7, value);
    dataChanged();
  }
  
  public final void setFontColor(int value) {
    int color = RGB2BGR(value);
    int index = this.grid.fontColors().indexOf(color);
    if (index < 0)
      if (this.grid.fontColors().count() < 63) {
        index = this.grid.fontColors().add(color);
      } else {
        index = 63;
      }  
    this.data[6] = (byte)(this.data[6] & 0xC0 | index & 0x3F);
    dataChanged();
  }
  
  public final void setFontHeight(int value) {
    setFontSize(value * 72 / 96);
  }
  
  public final void setFontItalic(boolean value) {
    setBit(7, 6, value);
    dataChanged();
  }
  
  public final void setFontName(String value) {
    int index = this.grid.fontNames().indexOfFont(value);
    if (index == -1)
      if (this.grid.fontNames().count() < 63) {
        index = this.grid.fontNames().addFont(value);
      } else {
        index = 0;
      }  
    this.data[4] = (byte)(this.data[4] & 0xC0 | index & 0x3F);
    dataChanged();
  }
  
  public final void setFontSize(int value) {
    int fontsize = value;
    int index = this.grid.fontSizes().indexOf(fontsize);
    if (index < 0)
      if (this.grid.fontSizes().count() < 63) {
        index = this.grid.fontSizes().add(fontsize);
      } else {
        index = 0;
      }  
    this.data[5] = (byte)(this.data[5] & 0xC0 | index & 0x3F);
    dataChanged();
  }
  
  public final void setFontStrikeOut(boolean value) {
    setBit(7, 5, value);
    dataChanged();
  }
  
  public final void setFontStyle(int fontStyle, boolean value) {
    if (fontStyle >= 0 && fontStyle < 4)
      setBit(1, fontStyle, value); 
    dataChanged();
  }
  
  public final void setFontUnderLine(boolean value) {
    setBit(7, 4, value);
    dataChanged();
  }
  
  public final void setHorzAlign(int value) {
    this.data[7] = (byte)(this.data[7] & 0xF8 | value & 0x7);
    dataChanged();
  }
  
  public final void setImeMode(int value) {
    this.data[11] = (byte)(this.data[11] & 0xF8 | value & 0x7);
    dataChanged();
  }
  
  public final void setIndent(int value) {
    this.data[1] = (byte)(this.data[1] & 0xF0 | value & 0xF);
    dataChanged();
  }
  
  public void setLEdgeColor(int value) {
    if (this.col == 0)
      return; 
    com.jiuqi.grid.GridCell cell = this.grid.getCellEx(this.col - 1, this.row);
    cell.setREdgeColor(value);
    this.grid.setCell(this.col - 1, this.row, cell);
  }
  
  public void setLEdgeStyle(int value) {
    if (this.col == 0)
      return; 
    com.jiuqi.grid.GridCell cell = this.grid.getCellEx(this.col - 1, this.row);
    cell.setREdgeStyle(value);
    this.grid.setCell(this.col - 1, this.row, cell);
  }
  
  public final void setMultiLine(boolean value) {
    setBit(8, 7, value);
    dataChanged();
  }
  
  public final void setREdgeColor(int value) {
    int color = RGB2BGR(value);
    int index = this.grid.edgeColors().indexOf(color);
    if (index < 0)
      if (this.grid.edgeColors().count() < 15) {
        index = this.grid.edgeColors().add(color);
      } else {
        index = 0;
      }  
    this.data[9] = (byte)(this.data[9] & 0xF | index << 4 & 0xF0);
    dataChanged();
  }
  
  public final void setREdgeStyle(int value) {
    this.data[9] = (byte)(this.data[9] & 0xF0 | value & 0xF);
    dataChanged();
  }
  
  public final void setShowFormat(String value) {}
  
  public final void setSilverHead(boolean value) {
    if (getBit(8, 6) == value)
      return; 
    setBit(8, 6, value);
    if (value) {
      setBackColor(-16777201);
      setBackStyle(28);
      setLEdgeColor(8421504);
      setREdgeColor(8421504);
      setTEdgeColor(8421504);
      setBEdgeColor(8421504);
    } else {
      setBackColor(-16777211);
      setBackStyle(0);
      setLEdgeColor(-16777201);
      setREdgeColor(-16777201);
      setTEdgeColor(-16777201);
      setBEdgeColor(-16777201);
    } 
    dataChanged();
  }
  
  public void setTEdgeColor(int value) {
    if (this.row == 0)
      return; 
    com.jiuqi.grid.GridCell cell = this.grid.getCellEx(this.col, this.row - 1);
    cell.setBEdgeColor(value);
    this.grid.setCell(this.col, this.row - 1, cell);
  }
  
  public void setTEdgeStyle(int value) {
    if (this.row == 0)
      return; 
    com.jiuqi.grid.GridCell cell = this.grid.getCellEx(this.col, this.row - 1);
    cell.setBEdgeStyle(value);
    this.grid.setCell(this.col, this.row - 1, cell);
  }
  
  public final void setVertAlign(int value) {
    this.data[8] = (byte)(this.data[8] & 0xF8 | value & 0x7);
    dataChanged();
  }
  
  public final void setVertText(boolean value) {
    setBit(8, 5, value);
    dataChanged();
  }
  
  public final void setWrapLine(boolean value) {
    setBit(8, 4, value);
    dataChanged();
  }
  
  public final void setBoolean(boolean value) {
    this.grid.setCellData(this.col, this.row, String.valueOf(value));
  }
  
  public final void setDate(Date value) {
    String str = (value == null) ? null : DateUtil.convertDateToString1(value);
    this.grid.setCellData(this.col, this.row, str);
  }
  
  public final void setDateTime(Date value) {
    setDate(value);
  }
  
  public final void setFloat(double value) {
    this.grid.setCellData(this.col, this.row, String.valueOf(value));
  }
  
  public final void setInteger(int value) {
    this.grid.setCellData(this.col, this.row, String.valueOf(value));
  }
  
  public final void setString(String value) {
    this.grid.setCellData(this.col, this.row, value);
  }
  
  public final void setTime(double value) {
    this.grid.setCellData(this.col, this.row, String.valueOf(value));
  }
  
  public final void setCellData(String value) {
    this.grid.setCellData(this.col, this.row, value);
  }
  
  public final void setColNum(int value) {
    this.col = value;
  }
  
  public final void setColWidth(int value) {
    this.grid.setColWidths(this.col, value);
  }
  
  public final void setHasConjection(boolean value) {}
  
  public final void setIsNull(boolean value) {
    this.grid.setCellData(this.col, this.row, value ? null : "");
  }
  
  public final void setRowHeight(int value) {
    this.grid.setRowHeights(this.row, value);
  }
  
  public final void setRowNum(int value) {
    this.row = value;
  }
  
  public final void setVisible(boolean value) {}
  
  public final String getCssClass() {
    return this.grid.getCellClass(this.col, this.row);
  }
  
  public final void setCssClass(String value) {
    this.grid.setCellClass(this.col, this.row, value);
  }
  
  public final String getImageReference() {
    String result = getCellData();
    if (result != null && result.indexOf(':') > 0) {
      String ext = result.substring(0, result.indexOf(':'));
      ext = ext.toUpperCase();
      if (ext.equals("JPG") || ext.equals("BMP") || ext.equals("GIF") || ext.equals("WMF") || ext.equals("PNG") || ext.equals("JPEG"))
        result = null; 
    } 
    return result;
  }
  
  public final void setImageReference(String ref) {
    setDataType(6);
    setCellData(ref);
  }
  
  public final String getImageType() {
    String data = getCellData();
    if (data == null)
      return null; 
    int pos = data.indexOf(':');
    if (pos >= 0) {
      String ext = data.substring(0, pos);
      return ext.toUpperCase();
    } 
    return null;
  }
  
  public final byte[] getImageData() {
    String data = getCellData();
    if (data == null)
      return null; 
    int pos = data.indexOf(':');
    if (pos >= 0) {
      String imgStr = data.substring(pos + 1);
      return Base64.base64ToByteArray(imgStr);
    } 
    return null;
  }
  
  public final void setImageData(byte[] data, String imgExtension) {
    String ext = (imgExtension == null || imgExtension.length() == 0) ? "JPG" : imgExtension.toUpperCase();
    String cellData = String.valueOf(ext) + ":" + Base64.byteArrayToBase64(data);
    setDataType(6);
    setCellData(cellData);
  }
  
  public final String[] getLinkInformation() {
    return parserLinkInformation(getCellData());
  }
  
  public static final String[] parserLinkInformation(String linkValue) {
    String[] linkInfo = new String[3];
    if (linkValue != null)
      if (linkValue.length() <= 5) {
        linkInfo[0] = linkValue;
      } else {
        String head = linkValue.substring(0, 5).toUpperCase();
        if (!head.equals("LINK:")) {
          linkInfo[0] = linkValue;
        } else {
          try {
            byte[] data = Base64.base64ToByteArray(linkValue.substring(5));
            MemStream stream = new MemStream();
            stream.writeBuffer(data, 0, data.length);
            stream.setPosition(0L);
            int count = stream.readInt();
            linkInfo[0] = stream.readString(count);
            count = stream.readInt();
            linkInfo[1] = stream.readString(count);
            count = stream.readInt();
            linkInfo[2] = stream.readString(count);
          } catch (Exception ex) {
            linkInfo[0] = linkValue;
            linkInfo[1] = null;
            linkInfo[2] = null;
            ex.printStackTrace();
          } 
        } 
      }  
    return linkInfo;
  }
  
  public final void setLinkInformation(String text, String reference, String target) {
    setDataType(9);
    String data = makeLinkInformation(text, reference, target);
    setCellData(data);
  }
  
  private static void writeString(Stream stream, String text) throws StreamException {
    if (text == null || text.length() == 0) {
      stream.writeInt(0);
    } else {
      byte[] data;
      try {
        data = text.getBytes(stream.getCharset());
      } catch (UnsupportedEncodingException unsupportedEncodingException) {
        data = text.getBytes();
      } 
      stream.writeInt(data.length);
      stream.writeBuffer(data, 0, data.length);
    } 
  }
  
  public static final String makeLinkInformation(String text, String reference, String target) {
    try {
      MemStream stream = new MemStream();
      writeString((Stream)stream, text);
      writeString((Stream)stream, reference);
      writeString((Stream)stream, target);
      String data = Base64.byteArrayToBase64(stream.getBytes());
      return "LINK:" + data;
    } catch (StreamException e) {
      e.printStackTrace();
      return null;
    } 
  }
  
  public final int getDataFlag() {
    return this.data[12] & 0xF;
  }
  
  public final void setDataFlag(int value) {
    this.data[12] = (byte)(this.data[12] & 0xF0 | value & 0xF);
    dataChanged();
  }
  
  public final int getDataFormat() {
    return this.data[12] >>> 4;
  }
  
  public final void setDataFormat(int value) {
    this.data[12] = (byte)(this.data[12] & 0xF | (value & 0xF) << 4);
    dataChanged();
  }
  
  public final int getDataProperty() {
    int r = this.data[14];
    r = (r << 8) + this.data[13];
    return r;
  }
  
  public final void setDataProperty(int value) {
    this.data[14] = (byte)(value >>> 8);
    this.data[13] = (byte)(value & 0xFF);
    dataChanged();
  }
  
  public final void setOutputKind(int value) {
    this.data[11] = (byte)(this.data[11] & 0xCF | value << 4 & 0x30);
    dataChanged();
  }
  
  public final String getScript() {
    return this.grid.getCellScript(this.col, this.row);
  }
  
  public final void setScript(String value) {
    this.grid.setCellScript(this.col, this.row, value);
  }
  
  public final int getWidth() {
    int width = 0;
    CellField cf = this.grid.expandCell(this.col, this.row);
    for (int i = cf.left; i <= cf.right; i++)
      width += this.grid.getColWidths(i); 
    return width;
  }
  
  public final int getHeight() {
    int height = 0;
    CellField cf = this.grid.expandCell(this.col, this.row);
    for (int i = cf.top; i <= cf.bottom; i++)
      height += this.grid.getRowHeights(i); 
    return height;
  }
  
  public String toString() {
    StringBuffer result = new StringBuffer();
    result.append("[");
    result.append(this.col);
    result.append(",");
    result.append(this.row);
    result.append("]:");
    result.append(getCellData());
    return result.toString();
  }
  
  public final String getEditText() {
    String storeText = getCellData();
    if (storeText == null)
      return null; 
    int dataType = getDataType();
    CellDataProperty cdp = new CellDataProperty(getDataType(), getDataFlag(), getEditMode(), getDataFormat(), getDataProperty());
    switch (dataType) {
      case 1:
        return (new TextCellProperty((CellDataPropertyIntf)cdp)).getEditText(storeText);
      case 2:
        return (new NumberCellProperty((CellDataPropertyIntf)cdp)).getEditText(storeText);
      case 3:
        return (new CurrencyCellProperty((CellDataPropertyIntf)cdp)).getEditText(storeText);
    } 
    return storeText;
  }
  
  public final void setEditText(String editText) {
    setCellData(parseEditText(editText));
  }
  
  public final String getShowText() {
    String storeText = getCellData();
    if (storeText == null)
      return null; 
    int dataType = getDataType();
    CellDataProperty cdp = new CellDataProperty(getDataType(), getDataFlag(), getEditMode(), getDataFormat(), getDataProperty());
    switch (dataType) {
      case 1:
        return (new TextCellProperty((CellDataPropertyIntf)cdp)).getShowText(storeText);
      case 2:
        return (new NumberCellProperty((CellDataPropertyIntf)cdp)).getShowText(storeText);
      case 3:
        return (new CurrencyCellProperty((CellDataPropertyIntf)cdp)).getShowText(storeText);
    } 
    return storeText;
  }
  
  public final void setShowText(String showText) {
    if (showText == null || showText.length() == 0) {
      setCellData("");
    } else {
      setCellData(parseShowText(showText));
    } 
  }
  
  public final String parseEditText(String editText) {
    int dataType = getDataType();
    CellDataProperty cdp = new CellDataProperty(getDataType(), getDataFlag(), getEditMode(), getDataFormat(), getDataProperty());
    switch (dataType) {
      case 1:
        return (new TextCellProperty((CellDataPropertyIntf)cdp)).parseEditText(editText);
      case 2:
        return (new NumberCellProperty((CellDataPropertyIntf)cdp)).parseEditText(editText);
      case 3:
        return (new CurrencyCellProperty((CellDataPropertyIntf)cdp)).parseEditText(editText);
    } 
    return editText;
  }
  
  public final String parseShowText(String showText) {
    int dataType = getDataType();
    CellDataProperty cdp = new CellDataProperty(getDataType(), getDataFlag(), getEditMode(), getDataFormat(), getDataProperty());
    switch (dataType) {
      case 1:
        return (new TextCellProperty((CellDataPropertyIntf)cdp)).parseShowText(showText);
      case 2:
        return (new NumberCellProperty((CellDataPropertyIntf)cdp)).parseShowText(showText);
      case 3:
        return (new CurrencyCellProperty((CellDataPropertyIntf)cdp)).parseShowText(showText);
    } 
    return showText;
  }
  
  public boolean equals(Object obj) {
    if (this == obj)
      return true; 
    if (this == null || obj == null)
      return false; 
    if (!(obj instanceof com.jiuqi.grid.GridCell))
      return false; 
    com.jiuqi.grid.GridCell gridCell = (com.jiuqi.grid.GridCell)obj;
    byte[] cmpData = gridCell.getPropData();
    if (this.data.length != cmpData.length)
      return false; 
    for (int i = 0; i < this.data.length; i++) {
      if (this.data[i] != cmpData[i])
        return false; 
    } 
    int srcCellLeftBorder = (this.col == 0) ? 0 : this.grid.getCellPropData(this.col - 1, this.row)[9];
    int srcCellTopBorder = (this.row == 0) ? 0 : this.grid.getCellPropData(this.col, this.row - 1)[10];
    int tarCellLeftBorder = (this.col == 0) ? 0 : this.grid.getCellPropData(gridCell.getColNum() - 1, gridCell.getRowNum())[9];
    int tarCellTopBorder = (this.row == 0) ? 0 : this.grid.getCellPropData(gridCell.getColNum(), gridCell.getRowNum() - 1)[10];
    if (srcCellLeftBorder != tarCellLeftBorder || srcCellTopBorder != tarCellTopBorder)
      return false; 
    return true;
  }
  
  public int hashCode() {
    int result = 0;
    for (int i = 0; i < this.data.length; i++) {
      if (i == 9) {
        result = 37 * result + ((this.col == 0) ? 0 : this.grid.getCellPropData(this.col - 1, this.row)[9]);
      } else if (i == 10) {
        result = 37 * result + ((this.row == 0) ? 0 : this.grid.getCellPropData(this.col, this.row - 1)[10]);
      } else {
        result = 37 * result + this.data[i];
      } 
    } 
    return result;
  }
  
  public final NumberCellPropertyIntf toNumberCell() {
    return (NumberCellPropertyIntf)new NumberCellProperty(this);
  }
  
  public final TextCellPropertyIntf toTextCell() {
    return (TextCellPropertyIntf)new TextCellProperty(this);
  }
  
  public final CurrencyCellPropertyIntf toCurrencyCell() {
    return (CurrencyCellPropertyIntf)new CurrencyCellProperty(this);
  }
}
